#include "Level.h"

void Level::displayBg(Rect camera, int xOffset, int yOffset)
{
    if(bg)
        bg->display(xOffset, yOffset); // bg doesn't need no stinkin' camera

    for(vector<Decoration*>::iterator it = bgDecs.begin(); it != bgDecs.end(); it++)
    {
        (*it)->display(xOffset, yOffset, camera);
    }

    for(vector<Platform*>::iterator it = platforms.begin(); it != platforms.end(); it++)
    {
        (*it)->display(xOffset - camera.x, yOffset-camera.y);
    }
}

void Level::displayFg(Rect camera, int xOffset, int yOffset)
{
    winZone.display(xOffset, yOffset, camera);

    for(vector<Trap*>::iterator it = traps.begin(); it != traps.end(); it++)
    {
        (*it)->display(xOffset - camera.x, yOffset-camera.y);
    }

    for(vector<Decoration*>::iterator it = fgDecs.begin(); it != fgDecs.end(); it++)
    {
        (*it)->display(xOffset, yOffset, camera);
    }
}

void Level::loadProperties(ifstream& in)
{
    string line, key, value;

    while(getline(in, line) && line != "end")
    {
        if(line == "" || line==" " || line[0]==';')
        {

        }
        else if(line[0]=='[')
        {
            // GET OUT GET OUT
            return;
        }
        key = line.substr(0, line.find_first_of(":"));
        value = line.substr(line.find_first_of(":") + 1);
        if(!value.empty() && value[0] == ' ')
            value = value.substr(1);

        if(key == "width")
        {
            width = atoi(value.c_str());
            cout << "level width set to " << width << "\n";
        }
        else if(key == "height")
        {
            height = atoi(value.c_str());
            cout << "level height set to " << height << "\n";
        }
        else if(key == "spawnx")
        {
            spawnX = atoi(value.c_str());
            cout << "spawn x set to " << spawnX << "\n";
        }
        else if(key == "spawny")
        {
            spawnY = atoi(value.c_str());
            cout << "spawn y set to " << spawnY << "\n";
        }
        else if(key == "winx")
        {
            winZone.setX(atoi(value.c_str()));
            cout << "win x set to " << winZone.getBounds().x << "\n";
        }
        else if(key == "winy")
        {
            winZone.setY(atoi(value.c_str()));
            cout << "win x set to " << winZone.getBounds().y << "\n";
        }
        else if(key == "winimg")
        {
            if(value.substr(8) != "sprites/")
                value = "sprites/" + value;

            winZone.setTexture(value);
            winZone.setFloating(true);
            winZone.setZ(0);
        }

        else if(key == "nextmap")
        {
            nextLevelName = value;
            cout << "next map set to " << nextLevelName << "\n";
        }
    }
}

void Level::loadBackground(ifstream& in)
{
    string line, key, value;

    while(getline(in, line) && line != "end")
    {
        if(line == "" || line==" " || line[0]==';')
        {

        }
        else if(line[0]=='[')
        {
            // GET OUT GET OUT
            return;
        }
        key = line.substr(0, line.find_first_of(":"));
        value = line.substr(line.find_first_of(":") + 1);
        if(!value.empty() && value[0] == ' ')
            value = value.substr(1);

        if(key == "img")
        {
            if(bg)
            {
                delete bg;
                bg = NULL;
            }

            if(value.substr(8) != "sprites/")
                value = "sprites/" + value;


            bg = new Image(value);
        }
    }
}


void Level::loadPlatform(ifstream& in)
{
    string line, key, value;
    Platform* plat = new Platform();

    while(getline(in, line) && line != "end")
    {
        if(line == "" || line==" " || line[0]==';')
        {

        }
        else if(line[0]=='[')
        {
            delete plat;
            return;
        }
        else
        {
            key = line.substr(0, line.find_first_of(":"));
            value = line.substr(line.find_first_of(":") + 1);
            if(!value.empty() && value[0] == ' ')
                value = value.substr(1);

            if(key == "img")
            {
                if(value.substr(8) != "sprites/")
                    value = "sprites/" + value;

                plat->setSprite(value);
            }
            else if(key == "xOffset")
            {
                plat->setXOffset(atoi(value.c_str()));
            }
            else if(key == "yOffset")
            {
                plat->setYOffset(atoi(value.c_str()));
            }
            else if(key == "x")
            {
                plat->setX(atoi(value.c_str()));
            }
            else if(key == "y")
            {
                plat->setY(atoi(value.c_str()));
            }
            else if(key == "w")
            {
                plat->setWidth(atoi(value.c_str()));
            }
            else if(key == "h")
            {
                plat->setHeight(atoi(value.c_str()));
            }
            else if(key == "flip" || key == "mirror" || key == "flipped")
            {
                if(value == "yes" || value == "TRUE" || value == "ON" || value == "YES")
                    plat->setFlip(true);
            }
        }
    }


    platforms.push_back(plat);

}


void Level::loadTrap(ifstream& in)
{
    string line, key, value;
    Trap* dec = new Trap();

    while(getline(in, line) && line != "end")
    {
        if(line == "" || line==" " || line[0]==';')
        {

        }
        else if(line[0]=='[')
        {
            delete dec;
            return;
        }
        else
        {
            key = line.substr(0, line.find_first_of(":"));
            value = line.substr(line.find_first_of(":") + 1);
            if(!value.empty() && value[0] == ' ')
                value = value.substr(1);

            if(key == "img")
            {
                if(value.substr(8) != "sprites/")
                    value = "sprites/" + value;

                dec->setSprite(value);
            }
            else if(key == "x")
            {
                dec->setX(atoi(value.c_str()));
            }
            else if(key == "y")
            {
                dec->setY(atoi(value.c_str()));
            }
            else if(key == "xfloating")
            {
                dec->setXFloating(atoi(value.c_str()));
            }
            else if(key == "yfloating")
            {
                dec->setYFloating(atoi(value.c_str()));
            }
            else if(key == "partition")
            {
                dec->partition(atoi(value.c_str()));
            }
            else if(key == "sfx")
            {
                dec->setLoopSound(value);
            }
        }
    }

    traps.push_back(dec);
}

void Level::loadDecoration(ifstream& in)
{
    string line, key, value;
    Decoration* dec = new Decoration();

    while(getline(in, line) && line != "end")
    {
        if(line == "" || line==" " || line[0]==';')
        {

        }
        else if(line[0]=='[')
        {
            delete dec;
            return;
        }
        else
        {
            key = line.substr(0, line.find_first_of(":"));
            value = line.substr(line.find_first_of(":") + 1);
            if(!value.empty() && value[0] == ' ')
                value = value.substr(1);

            if(key == "img")
            {
                if(value.substr(8) != "sprites/")
                    value = "sprites/" + value;

                dec->setTexture(value);
            }
            else if(key == "x")
            {
                dec->setX(atoi(value.c_str()));
            }
            else if(key == "y")
            {
                dec->setY(atoi(value.c_str()));
            }
            else if(key == "z")
            {
                dec->setZ(atoi(value.c_str()));
            }
            else if(key == "floating")
            {
                if(value == "yes" || value == "Yes" || value == "true" || value == "True")
                    dec->setFloating(true);
            }
            else if(key == "flip" || key == "mirror" || key == "flipped")
            {
                if(value == "yes" || value == "TRUE" || value == "ON" || value == "YES")
                    dec->setXFlip(true);
            }
        }
    }

    if(dec->getZ()<0)
        fgDecs.push_back(dec);
    else
        bgDecs.push_back(dec);
}

bool Level::loadFile(string filename)
{
    bgDecs.clear();
    platforms.clear();
    traps.clear();
    fgDecs.clear();

    if(filename.substr(5) != "data/")
        filename = "data/" + filename;

    cout << "[LEVEL] loading " << filename << "\n";

    ifstream levelFile(filename.c_str());
    if(levelFile)
    {
        string mode = "unknown", line;
        levelFile.seekg (0, ios::beg);
        while(getline(levelFile, line))
        {
            if(line == "" || line==" " || line[0]==';')
            {

            }
            else if(line[0]=='[')
            {
                mode = line.substr(1, line.find_first_of("]") - 1);
                if(mode == "Decoration")
                    loadDecoration(levelFile);
                else if(mode == "BG" || mode == "bg" || mode == "background" || mode == "Background")
                    loadBackground(levelFile);
                else if(mode == "Properties")
                    loadProperties(levelFile);
                else if(mode == "Platform")
                    loadPlatform(levelFile);
                else if(mode == "Trap")
                    loadTrap(levelFile);
            }
            else if(line == "end")
                mode = "unknown";
        }
        levelFile.close();
    }
    else
    {
        cout << "[LEVEL] ERROR: file failed to open!\n";
        return false;
    }

    cout << "[LEVEL] file loaded successfully.\n";
    return true;
}
