#include "Pony.h"

Pony::Pony()
{
    intendedXAcc = 0;
    grounded = false;
    dj = WubManager::Instance()->getDJ();
    jumpSound = WubManager::Instance()->requestSoundSource("sounds/jump.wav");
    jumpSound->setDefaultVolume(.5f);
    landSound = WubManager::Instance()->requestSoundSource("sounds/land.wav");
    landSound->setDefaultVolume(.2f);
}

Pony::Pony(string spritefile) : Object(spritefile) {
    intendedXAcc = 0;
    grounded = false;
    dj = WubManager::Instance()->getDJ();
    jumpSound = WubManager::Instance()->requestSoundSource("sounds/jump.wav");
    jumpSound->setDefaultVolume(.5f);
    landSound = WubManager::Instance()->requestSoundSource("sounds/land.wav");
    landSound->setDefaultVolume(.2f);
}

void Pony::ponySetup()
{
    sprite->partition(6 ,3);
    sprite->setFrameDelay(16);
    sprite->setOffset(-15, -9);
    hitbox.w = 34;
    hitbox.h = 46;
}


Pony::~Pony()
{

}

void Pony::jump()
{
    yAcc(-11*VELOCITY_SCALE);
    dj->play3D(jumpSound, irrklang::vec3df((hitbox.x + hitbox.w/2)/SOUNDSCALE, (hitbox.y + hitbox.h)/SOUNDSCALE, 0));
}

void Pony::setIntendedXAcc(int acc) {
    intendedXAcc = acc;
}

void Pony::setGrounded(bool grounded) {

    if(!this->grounded && grounded)
        dj->play3D(landSound, irrklang::vec3df((hitbox.x + hitbox.w/2)/SOUNDSCALE, (hitbox.y + hitbox.h)/SOUNDSCALE, 0));

    this->grounded = grounded;

    if(!grounded)
    {
         if(yVel >= 0)
        {
            sprite->setFrameDelay(16);
            sprite->setTrack(3); // jumping/falling
            sprite->setFrame(2); // jumping
        }
        else if(yVel < 0)
        {
            sprite->setFrameDelay(16);
            sprite->setTrack(3); // jumping/falling
            sprite->setFrame(1); // falling
        }
        friction = 0;
    }
    else
    {
        if(intendedXAcc)
        {
            sprite->setFrameDelay(4);
            sprite->setTrack(2); // trotting
        }
        else
        {
            sprite->setFrameDelay(16);
            sprite->setTrack(1); // standing
        }
    }
}

void Pony::display(int xOffset, int yOffset)
{
    if(sprite->currentTrack() != 3)
        sprite->nextFrame();

    sprite->display(hitbox.x + xOffset, hitbox.y + yOffset);
}

void Pony::faceRight(bool right)
{
    if(right)
        sprite->setXFlip(false);
    else
        sprite->setXFlip(true);
}

void Pony::act()
{
    xVel += intendedXAcc;
    if(!intendedXAcc)
        xBrake();
}


void Pony::xBrake()
{
    if(xVel > 0)
    {
        xVel -= friction;
        if(xVel < 0)
            xVel = 0;
    }
    else if(xVel < 0)
    {
        xVel += friction;
        if(xVel > 0)
            xVel = 0;
    }
}
