#include "Game.h"
#include "Menu.h"

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <sstream>

using namespace std;

StateMachine* stateMachine = NULL;
StateLoopIFC* currentActor = NULL;
Artist* artist = NULL;

void display()
{
    glClear( GL_COLOR_BUFFER_BIT );
    try
    {
        currentActor -> display();
        glutSwapBuffers();
    }
    catch(...){ cout << "CAUGHT AN ERROR IN DISPLAY: " << stateMachine->getGameStateString() << "\n"; }

}

void key(unsigned char key, int x, int y)
{
    try{currentActor -> key(key, x, y);}
    catch(...){ cout << "CAUGHT AN ERROR IN KEY : " << stateMachine->getGameStateString() << "\n"; }
}

void specialDown(int key, int x, int y)
{
    try{currentActor -> specialDown(key, x, y);}
    catch(...){ cout << "CAUGHT AN ERROR IN SPECIALDOWN : " << stateMachine->getGameStateString() << "\n"; }
}

void specialUp(int key, int x, int y)
{
    try{currentActor -> specialUp(key, x, y);}
    catch(...){ cout << "CAUGHT AN ERROR IN SPECIALUP : " << stateMachine->getGameStateString() << "\n"; }
}

void update(int value)
{
    try
    {
        currentActor -> update(value);
        // 60 fps = 1000/60 = 16.6
        glutTimerFunc(16, update, 0);
    }
    catch(...){ cout << "CAUGHT AN ERROR IN UPDATE : " << stateMachine->getGameStateString() << "\n"; }
}

void mouseButton(int button, int state, int x, int y)
{
    try
    {
        currentActor -> mouseButton(button, state, x, y);
    }
    catch(...){ cout << "CAUGHT AN ERROR IN MOUSEBUTTON : " << stateMachine->getGameStateString() << "\n"; }
}

void clickDrag(int x,int y)
{
    try
    {
        currentActor -> clickDrag(x, y);
    }
    catch(...){ cout << "CAUGHT AN ERROR IN CLICKDRAG : " << stateMachine->getGameStateString() << "\n"; }
}

void mouseMove(int x, int y)
{
    try
    {
        currentActor -> mouseMove(x, y);
    }
    catch(...){ cout << "CAUGHT AN ERROR IN MOUSEMOVE : " << stateMachine->getGameStateString() << "\n"; }
}

void reshape(int width, int height)
{
    if(width <=0)
        width = 1;
    if(height <= 0)
        height = 1;

    artist->setWindowWidth(width);
    artist->setWindowHeight(height);

    glViewport( 0, 0, artist->windowWidth(), artist->windowHeight());

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();

    glOrtho(0, artist->windowWidth(), artist->windowHeight(), 0, -1, 1);

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();

    try
    {
        currentActor -> reshape(width, height);
    }
    catch(...){ cout << "CAUGHT AN ERROR IN RESHAPE : " << stateMachine->getGameStateString() << "\n"; }
}

void readInitFile()
{
    bool fullscreen = false;

    ifstream settingsFile("settings.ini");
    if(settingsFile)
    {
        string mode = "default", line, key, value;
        settingsFile.seekg (0, ios::beg);
        while(getline(settingsFile, line))
        {
            if(line == "" || line==" " || line[0]==';')
            {

            }
            else if(line[0]=='[')
            {
                mode = line.substr(1, line.find_first_of("]") - 1);
            }
            else
            {
                key = line.substr(0, line.find_first_of("="));
                value = line.substr(line.find_first_of("=") + 1);
                if(key == "fullscreen" && (value == "1" || value == "true" || value == "on" || value == "yes" || value == "TRUE" || value == "ON" || value == "YES"))
                {
                    fullscreen = true;
                }
                else if(key == "width")
                {
                    artist->setWindowWidth(atoi(value.c_str()));
                }
                else if(key == "height")
                {
                    artist->setWindowHeight(atoi(value.c_str()));
                }
                else if(key == "apple sfx" && (value == "0" || value == "false" || value == "off" || value == "OFF" || value == "Off" || value == "no" || value == "NO"))
                {
                    WubManager::Instance()->setDefaultVolume(0);
                }
            }
        }
        settingsFile.close();
    }
    else
    {
        ofstream settingsFile("settings.ini");
        if(settingsFile)
        {
            settingsFile << "[window]\n";
            settingsFile << "fullscreen=" << fullscreen << "\n";
            settingsFile << "width=" << artist->windowWidth() << "\n";
            settingsFile << "height=" << artist->windowHeight() << "\n";
            settingsFile << "apple sfx=";
            if(WubManager::Instance()->getDefaultVolume() > 0)
                settingsFile << "on\n";
            else
                settingsFile << "off\n";
        }
        else
            cout << "ERROR: could not initialize settings.ini";
        settingsFile.close();
    }

    cout << "[MAIN] Screen: " << artist->windowWidth() << "x" << artist->windowHeight();
    fullscreen?cout << ", fullscreen\n":cout<<", windowed\n";

    glutInitWindowSize(artist->windowWidth(), artist->windowHeight());
    glutInitWindowPosition(0,0);
    glutCreateWindow("Chaos In The Sky");

    if(fullscreen)
    {
        stringstream gameMode;
        gameMode << artist -> windowWidth() << "x" << artist->windowHeight() << ":32@60";
        glutGameModeString(gameMode.str().c_str());

        if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))
            glutEnterGameMode();
    }

    glutSetKeyRepeat(GLUT_KEY_REPEAT_OFF);
    glutReshapeFunc(reshape);
    glutDisplayFunc(display);
    glutKeyboardFunc(key);
    glutSpecialFunc(specialDown);
    glutSpecialUpFunc(specialUp);
    glutMouseFunc(mouseButton);
    glutMotionFunc(clickDrag);
    glutPassiveMotionFunc(mouseMove);
    glutTimerFunc(0,update,0);
}

void initGL()
{
    artist = Artist::Instance();
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);

    readInitFile();

    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

    glEnable( GL_TEXTURE_2D ); // Need this to display a texture

    glViewport( 0, 0, artist->windowWidth(), artist->windowHeight() );

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();

    glOrtho(0, artist->windowWidth(), artist->windowHeight(), 0, -1, 1);

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
}

void enterGlutLoop()
{
    GameState* current = stateMachine->getGameStatePtr();
    GameState original = *current;
    while(*current == original)
    {
        glutMainLoopEvent();
    }
}

inline void redirectCout(streambuf *&backup, ofstream &filestr)
{
    streambuf *psbuf;

    backup = cout.rdbuf();     // back up cout's streambuf

    if(!filestr)
        return;

    psbuf = filestr.rdbuf();
    cout.rdbuf(psbuf);
}

int main(int argc, char *argv[])
{
#if RELEASE
    streambuf *coutBackup;
    ofstream filestr("release.log");
    redirectCout(coutBackup, filestr);
#endif
    bool quit = false;

    glutInit(&argc, argv);

    initGL();

    srand ( time(NULL) );

    stateMachine = new StateMachine();

    Menu mainMenu(stateMachine);
    Game game(stateMachine);

    currentActor = &mainMenu;



    while(!quit)
    {
        switch(stateMachine->getGameState())
        {
            case MENU:     mainMenu.assumeControl(currentActor); enterGlutLoop(); break;
            case NEWGAME:  if(!game.newGame()) { stateMachine->setGameState(MENU); cout << ">MENU newGame failed.\n"; break; }
            case CONTINUE: game.assumeControl(currentActor);     enterGlutLoop(); break;
            case GAMEDONE: mainMenu.setBg("sprites/win_screen.png"); stateMachine->setGameState(MENU); break;

            case QUIT: quit = true; break;

            default: stateMachine->setGameState(QUIT); cout <<"Exiting main loop on unknown state!\n";
        }
    }

#if RELEASE
    cout.rdbuf(coutBackup);        // restore cout's original streambuf
    filestr.close();
#endif

    Artist::Instance()->freeTextures();

    delete stateMachine;

    return 0;
}
